import request from "superagent";
import Promise from "promise-polyfill";
import hwSdk from "./hwSdk";

let language = "en";

function internalValidateResponse(err, res, callback) {
    if (err) {
        callback(err);
    }
    if (res) {
        if (res.type && res.type !== "application/json") {
            callback(new Error(
                `Response type is not application/json it is ${res.type}`));
        } else if (!res.ok) {
            callback(res.body);
        } else {
            callback(null, res.body);
        }
    }
}

function internalProcessRequest(req,
    { data, authenticationToken, callback, files }) {
    if (files !== undefined) {
        files.forEach((file) => {
            req.attach(file.partName, file.fileData);
        });
        req.field("data", JSON.stringify(data));
    } else {
        req.set("Content-Type", "application/json");
        req.send(data);
    }

    req
        .set("Accept-Language", language)
        .set("Accept", "application/json")
        .set("Authorization", `Bearer ${authenticationToken}`);
    req.end((err, res) => internalValidateResponse(err, res, callback));
}

export default class HyperwalletTransaction {
    static init(locale) {
        language = locale;
    }

    static get({ url, authenticationToken, callback }) {
        request.get(url)
            .set("Accept-Language", language)
            .set("Content-Type", "application/json")
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${authenticationToken}`)
            .end((err, res) => internalValidateResponse(err, res, callback));
    }

    static post({
        url, data, authenticationToken, callback, files,
    }) {
        const postRequest = request.post(url);
        internalProcessRequest(postRequest,
            {
                data, authenticationToken, callback, files,
            });
    }

    static put({
        url, data, authenticationToken, callback, files,
    }) {
        const putRequest = request.put(url);
        internalProcessRequest(putRequest,
            {
                data, authenticationToken, callback, files,
            });
    }

    static postGraphQl({ query, callback }) {
        hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
            if (error) {
                callback(error);
            } else {
                const graphqlQuery = typeof query === "function"
                    ? query(hyperwalletConfiguration.walletToken) : query;

                HyperwalletTransaction.post({
                    url: hyperwalletConfiguration.graphqlUri,
                    data: graphqlQuery,
                    authenticationToken: hyperwalletConfiguration.authenticationToken,
                    callback,
                });
            }
        });
    }

    static postGraphQlByPromise(query) {
        return new Promise((resolve, reject) => {
            hwSdk.getHyperwalletConfiguration((error, hyperwalletConfiguration) => {
                if (error) {
                    reject(error);
                } else {
                    const callback = (err, res) => (err ? reject(err) : resolve(
                        res));
                    const graphqlQuery = typeof query === "function" ?
                        query(hyperwalletConfiguration.walletToken) : query;

                    HyperwalletTransaction.post({
                        url: hyperwalletConfiguration.graphqlUri,
                        data: graphqlQuery,
                        authenticationToken: hyperwalletConfiguration.authenticationToken,
                        callback,
                    });
                }
            });
        });
    }
}
