import request from "superagent";
import sinon from "sinon";
import HyperwalletTransaction from "../HyperwalletTransaction";
import hwSdk from "../hwSdk";

const url = "url";
const authenticationToken = "authenticationToken";
const data = "data";
const filesData = [
    {
        partName: "partName",
        fileData: "fileData",
    },
];
const error = "errorDetails";
const type = "application/json";

const testCases = [
    {
        desc: "test description: error; no response",
        err: error,
        res: null,
    },
    {
        desc: "test description: response with application/json type",
        err: null,
        res: {
            body: "responseBody",
            type,
            ok: false,
        },
    },
    {
        desc: "test description: response with other type and ok false",
        err: null,
        res: {
            body: "responseBody",
            type,
            ok: false,
        },
    },
    {
        desc: "test description: response with ok true",
        err: null,
        res: {
            body: "responseBody",
            type,
            ok: true,
        },
    },
];

const hyperwalletConfiguration = {
    walletToken: "usr-token",
    graphqlUri: "http://localhost/graphql",
    authenticationToken: "auth-token",
};


describe("HyperwalletTransaction.js", () => {
    let requestStub;
    let errorSpy;
    let successSpy;

    const beforeTest = (() => {
        errorSpy = sinon.spy(console, "warn");
        successSpy = sinon.spy(console, "log");
    });

    const afterTest = (() => {
        errorSpy.restore();
        successSpy.restore();
    });

    beforeEach(() => {
        requestStub = sinon.stub(request.Request.prototype, "end");
    });

    afterEach(() => {
        requestStub.restore();
    });

    describe("rest", () => {
        const callback = (err, res) => {
            if (err) {
                console.warn(err);
            }
            if (res) {
                console.log(res);
            }
        };

        const verify = (test) => {
            if (test.err && !test.res) {
                sinon.assert.calledWith(errorSpy, test.err);
                sinon.assert.notCalled(successSpy);
            } else if (test.res.type && test.res.type !== type) {
                sinon.assert.called(errorSpy);
                sinon.assert.notCalled(successSpy);
            } else if (!test.res.ok) {
                sinon.assert.calledWith(errorSpy, test.res.body);
                sinon.assert.notCalled(successSpy);
            } else {
                sinon.assert.notCalled(errorSpy);
                sinon.assert.calledWith(successSpy, test.res.body);
            }
        };

        let attachStub;
        testCases.forEach(test => {
            it(`get - ${test.desc}`, () => {
                beforeTest();
                requestStub.callsFake(cb => {
                    cb(test.err, test.res);
                });
                HyperwalletTransaction.get(
                    { url, authenticationToken, callback });
                verify(test);
                afterTest();
            });

            it(`post - ${test.desc}`, () => {
                beforeTest();
                requestStub.callsFake(cb => {
                    cb(test.err, test.res);
                });
                attachStub = sinon.stub(request.Request.prototype, "attach");
                [filesData, undefined].forEach(files => {
                    const jsonSpy = sinon.spy(JSON, "stringify");
                    HyperwalletTransaction.post(
                        { url, data, authenticationToken, callback, files });
                    files !== undefined ?
                        sinon.assert.called(jsonSpy) :
                        sinon.assert.notCalled(jsonSpy);
                    verify(test);
                    jsonSpy.restore();
                });
                attachStub.restore();
                afterTest();
            });

            it(`put - ${test.desc}`, () => {
                beforeTest();
                requestStub.callsFake(cb => {
                    cb(test.err, test.res);
                });
                attachStub = sinon.stub(request.Request.prototype, "attach");
                [filesData, undefined].forEach(files => {
                    const jsonSpy = sinon.spy(JSON, "stringify");
                    HyperwalletTransaction.put(
                        { url, data, authenticationToken, callback, files });
                    files !== undefined ?
                        sinon.assert.called(jsonSpy) :
                        sinon.assert.notCalled(jsonSpy);
                    verify(test);
                    jsonSpy.restore();
                });
                attachStub.restore();
                afterTest();
            });
        });
    });

    describe("graphql", () => {
        let stub;
        let querySpy;
        let postSpy;

        beforeEach(() => {
            beforeTest();
            querySpy = sinon.spy(console, "info");
            postSpy = sinon.spy(HyperwalletTransaction, "post");
        });

        afterEach(() => {
            afterTest();
            querySpy.restore();
            postSpy.restore();
        });

        const query = (text) => console.info(text);
        const queryString = "query";

        const callback = (err, res) => {
            if (err) {
                console.warn(err);
            } else {
                console.log(res);
            }
        };

        const configError = "configError";

        describe("success", () => {
            beforeEach(() => {
                stub = sinon.stub(hwSdk, "getHyperwalletConfiguration")
                    .callsFake(cb => {
                        cb(null, hyperwalletConfiguration);
                    });
            });

            afterEach(() => {
                stub.restore();
            });

            it("postGraphQl - query function - should return success"
                + " response", () => {
                HyperwalletTransaction.postGraphQl({ query, callback });
                sinon.assert.notCalled(errorSpy);
                sinon.assert.calledWith(querySpy,
                    hyperwalletConfiguration.walletToken);
                sinon.assert.called(postSpy);
            });

            it("postGraphQl - query string - should return success"
                + " response", () => {
                HyperwalletTransaction.postGraphQl(
                    { query: queryString, callback });
                sinon.assert.notCalled(errorSpy);
                sinon.assert.notCalled(querySpy);
                sinon.assert.called(postSpy);
            });

            it("postGraphQlByPromise - should return success response", () => {
                const resolve = (res) => console.log(res);
                HyperwalletTransaction.postGraphQlByPromise(query)
                    .then((res) => resolve(res));
                sinon.assert.notCalled(errorSpy);
                sinon.assert.called(postSpy);
            });
        });

        describe("failure", () => {
            beforeEach(() => {
                stub = sinon.stub(hwSdk, "getHyperwalletConfiguration");
            });

            afterEach(() => {
                stub.restore();
            });

            it("postGraphQl - query function - should return error response",
                () => {
                    stub.callsFake(cb => {
                        cb(configError, hyperwalletConfiguration);
                    });
                    HyperwalletTransaction.postGraphQl({ query, callback });
                    sinon.assert.calledWith(errorSpy, configError);
                    sinon.assert.notCalled(querySpy);
                    sinon.assert.notCalled(postSpy);
                });

            it("postGraphQl - query string - should return error"
                + " response", () => {
                stub.callsFake(cb => {
                    cb(configError, hyperwalletConfiguration);
                });
                HyperwalletTransaction.postGraphQl(
                    { query: queryString, callback });
                sinon.assert.calledWith(errorSpy, configError);
                sinon.assert.notCalled(querySpy);
                sinon.assert.notCalled(postSpy);
            });

            it("postGraphQlByPromise - should return error response", () => {
                const reject = (err) => console.warn(err);
                stub.callsFake(() => callback(configError));
                HyperwalletTransaction.postGraphQlByPromise(query)
                    .then((err) => reject(err));
                sinon.assert.calledWith(errorSpy, configError);
                sinon.assert.notCalled(postSpy);
            });
        });
    });
});
